<?php

namespace TVn7\EventBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\ParamConverter;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;

use TVn7\EventBundle\Entity\Comment;
use TVn7\EventBundle\Entity\Clip;
use TVn7\EventBundle\Entity\Event;
use TVn7\EventBundle\Entity\Master;
use TVn7\EventBundle\Entity\Music;
use TVn7\EventBundle\Entity\Sequence;
use TVn7\UserBundle\Entity\Membre;

use Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException;

use TVn7\EventBundle\Form\ClipType;
use TVn7\EventBundle\Form\MusicType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;

use TVn7\EventBundle\Security\ClipVoter;
use TVn7\EventBundle\Security\EventVoter;

use Symfony\Component\Routing\Generator\UrlGenerator;

/**
 * @Security("has_role('ROLE_EVENT_USER')")
 */
class ClipController extends Controller
{
    
    /**
     * @param Request $request
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|Response
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function newAction(Request $request, Event $event)
    {
        $this->denyAccessUnlessGranted(EventVoter::CREATE_CLIP, $event);

        $clip = new Clip();
        $clip->addRespoMontage($this->getUser());
        $clip->setEvent($event);

        $framerate = $this->getParameter("tvn7_event.config")["framerate"];
        $codec = $this->getParameter("tvn7_event.config")["codec"];
        $width = $this->getParameter("tvn7_event.config")["video_width"];
        $height = $this->getParameter("tvn7_event.config")["video_height"];
        $form = $this->createForm(ClipType::class, $clip, [
            "annotation" => $this->isGranted(ClipVoter::ANNOTATE, $clip),
            "framerate" => $framerate,
            "creation" => true
        ]);

        $music = new Music();
        $musicform = $this->createForm(MusicType::class, $music);
        
        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);

            if ($form->isValid()) {
                $clip->setCreatedAt(new \DateTime());
                $em = $this->getDoctrine()->getManager();
                $em->persist($clip);
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");
                $notifier->notify(
                        $clip, "Nouveau clip",
                        sprintf("%s a créé le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                        $this->generateUrl("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );
                if ($clip->isFinished()) {
                    $notifier->notify(
                          $event, "Clip terminé",
                          sprintf("%s a marqué comme monté le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                          $this->generateUrl("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()], UrlGenerator::ABSOLUTE_URL)
                    );
                }

                $this->addFlash('success', 'Le clip a bien été créé.');

                return $this->redirectToRoute("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()]);
            }
        }
        return $this->render('TVn7EventBundle:Clip:new.html.twig', array(
            'form' => $form->createView(),
            'clip' => $clip,
            'musicform' => $musicform->createView(),
            'framerate' => $framerate,
            'codec' =>$codec,
            'width' =>$width,
            'height' =>$height,
        ));
    }

    private function getRouteForAction(Clip $clip, $from_event) {
        if($from_event == 1) {
            $route = $this->redirectToRoute("_tdb_event_show", ["slug"=> $clip->getEvent()->getSlug()]);
        } else {
            $route = $this->redirectToRoute("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()]);
        }

        return $route;
    }

    /**
     *
     * @param Clip $clip
     * @param Request $request
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|Response
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function editAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        $framerate = $this->getParameter("tvn7_event.config")["framerate"];
        $codec = $this->getParameter("tvn7_event.config")["codec"];
        $width = $this->getParameter("tvn7_event.config")["video_width"];
        $height = $this->getParameter("tvn7_event.config")["video_height"];

        $from_event = $request->get('from_event');

        $form = $this->createForm(ClipType::class, $clip, [
            "annotation" => $this->isGranted(ClipVoter::ANNOTATE, $clip),
            "framerate" => $framerate,
            "from_event" => $from_event
        ]);

        $music = new Music();
        $musicform = $this->createForm(MusicType::class, $music);
        
        if ($request->getMethod() == 'POST') {
            $form->handleRequest($request);

            if ($form->isValid()) {
                $clip->setEditedAt(new \DateTime());
                $em = $this->getDoctrine()->getManager();
                $em->persist($clip);
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");

                $notifier->notify(
                        $clip, "Clip édité",
                        sprintf("%s a édité le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                        $this->generateUrl("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );

                $this->addFlash('success', 'Le clip a bien été modifié.');

                return $this->getRouteForAction($clip, $form->get('from_event')->getData());
            }
        }
        return $this->render('TVn7EventBundle:Clip:edit.html.twig', array(
            'form' => $form->createView(),
            'clip' => $clip,
            'musicform' => $musicform->createView(),
            'framerate' => $framerate,
            'codec' =>$codec,
            'width' =>$width,
            'height' =>$height, 
        ));
    }

    /**
     * 
     * @param Clip $clip
     * @return Response
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function showAction(Event $event, Clip $clip) {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::SHOW, $clip);
        
        foreach($clip->getMusics() as $music) {
            $clips = $event->getClipsNoDel()->filter(function($p) use ($music, $clip) {
                return ($p->getMusics()->contains($music) && ($p !== $clip));
            });
            if ($clips->count() > 0) {
                $clipsli="";
                foreach($clips as $c) {
                    $clipsli.="<li><a href=\"" .
                            $this->generateUrl("_tdb_event_clip_show",
                                    ["event"=> $c->getEvent()->getSlug(),
                                        "clip" => $c->getSlug()])
                            . "\">"
                            . $c . "</a></li>";
                }
                $this->addFlash("warning", $music . " est déjà utilisée dans : <ul>" . $clipsli . "</ul>");
            }
        }
        
        return $this->render('TVn7EventBundle:Clip:show.html.twig',[
            'clip' => $clip,
        ]);
    }  
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function deleteAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::DELETE, $clip);

        if($clip->getCanceled()) {
            if($clip->isDeletableSafely()) {
                $em = $this->getDoctrine()->getManager();
                $clip->setEditedAt(new \DateTime());
                $clip->setDeleted(true);
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");

                $notifier->notify(
                        $clip, "Clip supprimé",
                        sprintf("%s a supprimé le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                        $this->generateUrl("_tdb_event_show", ["slug"=> $event->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );

                $this->addFlash('success', 'Le clip a bien été supprimé.');
                return $this->redirectToRoute("_tdb_event_show", ["slug" => $event->getSlug()]);
            } else {
                $this->addFlash('danger', 'Le clip n\'a pas été supprimé car il est utilisé dans '.$clip->getSequences()->count(). ' séquences(s).');
            }
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été supprimé car il n\'est pas marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function shootAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }
        
        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        if($clip->getCanceled() == false) {
            $em=$this->getDoctrine()->getManager();
            $clip->setShoted(true);
            $clip->setEditedAt(new \DateTime());
            $em->flush();

            $this->addFlash('success', 'Le clip a été marqué comme tourné.');
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme tourné car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }

    /**
     *
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function takeAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        if($clip->getCanceled() == false) {
            if($clip->getRespoMontage()->count() == 0) {
                $em=$this->getDoctrine()->getManager();
                $clip->addRespoMontage($this->getUser());
                $clip->setEditedAt(new \DateTime());
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");
                $notifier->notify(
                        $clip, "Clip attribué",
                        sprintf("%s s'est attribué le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                        $this->generateUrl("_tdb_event_show", ["slug"=> $event->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );

                $this->addFlash('success', 'Vous avez été nommé responsable montage de ce clip.');
            } else {
                $this->addFlash('danger', 'Le clip a déjà un ou des responsables.');
            }
        } else {
            $this->addFlash('danger', 'Vous ne pouvez pas modifier un clip annulé !');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function unShootAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        if($clip->getCanceled() == false) {
            if($clip->getValidatedAt() == null) {
                $em=$this->getDoctrine()->getManager();
                $clip->setShoted(false);
                $clip->setDerushed(false);
                $clip->setFinishedAt(null);
                $clip->setEditedAt(new \DateTime());
                $em->flush();

                $this->addFlash('success', 'Le clip a été marqué comme non tourné.');
            } else {
                $this->addFlash('danger', 'Le clip n\'a pas été marqué comme non tourné car il est marqué comme validé.');
            }
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme tourné car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function derushAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        if($clip->getCanceled() == false) {
            $em=$this->getDoctrine()->getManager();
            $clip->setShoted(true);
            $clip->setDerushed(true);
            $clip->setEditedAt(new \DateTime());
            $em->flush();

            $this->addFlash('success', 'Le clip a été marqué comme dérushé.');
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme dérushé car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function unDerushAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        if($clip->getCanceled() == false) {
            if($clip->getValidatedAt() == null) {
                $em=$this->getDoctrine()->getManager();
                $clip->setDerushed(false);
                $clip->setFinishedAt(null);
                $clip->setEditedAt(new \DateTime());
                $em->flush();

                $this->addFlash('success', 'Le clip a été marqué comme non dérushé.');
            } else {
                $this->addFlash('danger', 'Le clip n\'a pas été marqué comme non dérushé car il est marqué comme validé.');
            }
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme dérushé car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
        
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function finishAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        if($clip->getCanceled() == false) {
            $em=$this->getDoctrine()->getManager();
            $clip->setShoted(true);
            $clip->setDerushed(true);
            $clip->setFinishedAt(new \DateTime());
            $clip->setEditedAt(new \DateTime());
            $em->flush();

            $notifier = $this->get("tvn7.event_notifier");
            $notifier->notify(
                    $event, "Clip terminé",
                    sprintf("%s a marqué comme monté le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                    $this->generateUrl("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()], UrlGenerator::ABSOLUTE_URL)
            );

            $this->addFlash('success', 'Le clip a été marqué comme monté.');
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme monté car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function unFinishAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        if($clip->getCanceled() == false) {
            if($clip->getValidatedAt() == null) {
                $em=$this->getDoctrine()->getManager();
                $clip->setFinishedAt(null);
                $clip->setEditedAt(new \DateTime());
                $em->flush();

                $this->addFlash('success', 'Le clip a été marqué comme non monté.');
            } else {
                $this->addFlash('danger', 'Le clip n\'a pas été marqué comme non monté car il est marqué comme validé.');
            }
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme monté car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
        
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function validateAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::VALIDATE, $clip);

        if($clip->getCanceled() == false) {
            if($clip->isFinished() == true) {
                $em=$this->getDoctrine()->getManager();
                $clip->setValidatedAt(new \DateTime());
                $clip->setEditedAt(new \DateTime());
                $em->flush();

                $notifier = $this->get("tvn7.event_notifier");

                $notifier->notify(
                        $clip, "Clip validé",
                        sprintf("%s a validé le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                        $this->generateUrl("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()], UrlGenerator::ABSOLUTE_URL)
                );

                $this->addFlash('success', 'Le clip a été marqué comme validé.');
            } else {
                $this->addFlash('danger', 'Le clip n\'a pas été marqué comme validé car il n\'est pas terminé.');
            }            
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme validé car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function unValidateAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::VALIDATE, $clip);

        if($clip->getCanceled() == false) {
            $em=$this->getDoctrine()->getManager();
            $clip->setValidatedAt(null);
            $clip->setEditedAt(new \DateTime());
            $em->flush();

            $this->addFlash('success', 'Le clip a été marqué comme non validé.');
        } else {
            $this->addFlash('danger', 'Le clip n\'a pas été marqué comme validé car il est marqué comme annulé.');
        }
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function cancelAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);
        
        $em=$this->getDoctrine()->getManager();
        $clip->setCanceled(true);
        $clip->setEditedAt(new \DateTime());
        $em->flush();

        $notifier = $this->get("tvn7.event_notifier");

        $notifier->notify(
                $clip, "Clip annulé",
                sprintf("%s a annulé le clip « %s » dans l'évènement « %s »", $this->getUser(), $clip->getName(), $event->getName()),
                $this->generateUrl("_tdb_event_clip_show", ["event"=> $clip->getEvent()->getSlug(),"clip" => $clip->getSlug()], UrlGenerator::ABSOLUTE_URL)
        );

        $this->addFlash('success', 'Le clip a été marqué comme annulé.');
           
        return $this->getRouteForAction($clip, $request->get('from_event'));
    }
    
    /**
     * 
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     * 
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function unCancelAction(Event $event, Clip $clip, Request $request)
    {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::EDIT, $clip);

        $em=$this->getDoctrine()->getManager();
        $clip->setCanceled(false);
        $clip->setEditedAt(new \DateTime());
        $em->flush();

        $this->addFlash('success', 'Le clip a été marqué comme non annulé.');

        return $this->getRouteForAction($clip, $request->get('from_event'));
    }

    /**
     *
     * @param Clip $clip
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @ParamConverter("event", class="TVn7EventBundle:Event", options={"mapping" : {"event" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     * @ParamConverter("clip", class="TVn7EventBundle:Clip", options={"mapping" : {"clip" : "slug"}, "repository_method" = "findBySlugNoDeleted"})
     */
    public function seqCommentAction(Request $request, Event $event, Clip $clip) {
        if($clip->getEvent() !== $event) {
            throw $this->createNotFoundException();
        }

        $this->denyAccessUnlessGranted(ClipVoter::ANNOTATE, $clip);

        $data = [];
        switch($request->getMethod()) {
            case "GET" :
                $data["success"] = true;
                $data["comment"] = $clip->getSeqComment();
                break;
            case "POST" :
                $clip->setSeqComment($request->get("comment"));
                $validator = $this->get('validator');
                $errors = $validator->validate($clip);
                if(count($errors) == 0) {
                    $this->getDoctrine()->getManager()->flush();
                    $data["success"] = true;
                } else {
                    $data["success"] = false;
                    $data["error"] = (string) $errors;
                }
                break;
            default:
                throw new \Symfony\Component\Routing\Exception\MethodNotAllowedException();
        }

        $response = new JsonResponse($data, Response::HTTP_OK);
        return $response;
    }
}
